@@ -4,17 +4,12 @@ use warnings;
 use strict;
 use 5.008003;
 
-our $VERSION = '1.654';
+our $VERSION = '1.709';
 
-use Term::Choose::Constants qw( :screen :linux );
+use Term::Choose::Constants qw( :linux :keys TERM_READKEY );
+use Term::Choose::Screen    qw( hide_cursor show_cursor normal );
 
 
-my $Term_ReadKey; # declare but don't assign a value!
-BEGIN {
-    if ( eval { require Term::ReadKey; 1 } ) {
-        $Term_ReadKey = 1;
-    }
-}
 my $Stty = '';
 
 
@@ -25,7 +20,7 @@ sub new {
 
 sub _getc_wrapper {
     my ( $timeout ) = @_;
-    if ( $Term_ReadKey ) {
+    if ( TERM_READKEY ) {
         return Term::ReadKey::ReadKey( $timeout );
     }
     else {
@@ -41,12 +36,12 @@ sub __get_key_OS {
     if ( $c1 eq "\e" ) {
         my $c2 = _getc_wrapper( 0.10 );
         if    ( ! defined $c2 ) { return KEY_ESC; } # unused
-        #elsif ( $c2 eq 'A' ) { return VK_UP; }     vt 52
-        #elsif ( $c2 eq 'B' ) { return VK_DOWN; }
-        #elsif ( $c2 eq 'C' ) { return VK_RIGHT; }
-        #elsif ( $c2 eq 'D' ) { return VK_LEFT; }
-        #elsif ( $c2 eq 'H' ) { return VK_HOME; }
-         elsif ( $c2 eq 'O' ) {
+        elsif ( $c2 eq 'A' ) { return VK_UP; }     #vt 52
+        elsif ( $c2 eq 'B' ) { return VK_DOWN; }
+        elsif ( $c2 eq 'C' ) { return VK_RIGHT; }
+        elsif ( $c2 eq 'D' ) { return VK_LEFT; }
+        elsif ( $c2 eq 'H' ) { return VK_HOME; }
+        elsif ( $c2 eq 'O' ) {
             my $c3 = _getc_wrapper( 0 );
             if    ( $c3 eq 'A' ) { return VK_UP; }
             elsif ( $c3 eq 'B' ) { return VK_DOWN; }
@@ -71,34 +66,16 @@ sub __get_key_OS {
             elsif ( $c3 =~ m/^[0-9]$/ ) {
                 my $c4 = _getc_wrapper( 0 );
                 if ( $c4 eq '~' ) {
+                    # 1 = VK_HOME
                     if    ( $c3 eq '2' ) { return VK_INSERT; }
                     elsif ( $c3 eq '3' ) { return VK_DELETE; }
+                    # 4 = VK_END
                     elsif ( $c3 eq '5' ) { return VK_PAGE_UP; }
                     elsif ( $c3 eq '6' ) { return VK_PAGE_DOWN; }
                     else {
                         return NEXT_get_key;
                     }
                 }
-                elsif ( $c4 =~ m/^[;0-9]$/ ) { # response to "\e[6n"
-                    my $abs_curs_y = $c3;
-                    my $ry = $c4;
-                    while ( $ry =~ m/^[0-9]$/ ) {
-                        $abs_curs_y .= $ry;
-                        $ry = _getc_wrapper( 0 );
-                    }
-                    return NEXT_get_key if $ry ne ';';
-                    my $abs_curs_x = '';
-                    my $rx = _getc_wrapper( 0 );
-                    while ( $rx =~ m/^[0-9]$/ ) {
-                        $abs_curs_x .= $rx;
-                        $rx = _getc_wrapper( 0 );
-                    }
-                    if ( $rx eq 'R' ) {
-                        #$self->{abs_cursor_x} = $abs_curs_x; # unused
-                        $self->{abs_cursor_y} = $abs_curs_y;
-                    }
-                    return NEXT_get_key;
-                }
                 else {
                     return NEXT_get_key;
                 }
@@ -110,7 +87,7 @@ sub __get_key_OS {
                 my $y          = ord( _getc_wrapper( 0 ) ) - 32;
                 my $button = $self->__mouse_event_to_button( $event_type );
                 return NEXT_get_key if $button == NEXT_get_key;
-                return [ $self->{abs_cursor_y}, $button, $x, $y ];
+                return [ $button, $x, $y ];
             }
             elsif ( $c3 eq '<' && $mouse ) {  # SGR 1006
                 my $event_type = '';
@@ -135,7 +112,7 @@ sub __get_key_OS {
                 return NEXT_get_key if $button_released;
                 my $button = $self->__mouse_event_to_button( $event_type );
                 return NEXT_get_key if $button == NEXT_get_key;
-                return [ $self->{abs_cursor_y}, $button, $x, $y ];
+                return [ $button, $x, $y ];
             }
             else {
                 return NEXT_get_key;
@@ -177,7 +154,7 @@ sub __set_mode {
     $self->{mouse}       = $config->{mouse};        # so options passed with $config are
     $self->{hide_cursor} = $config->{hide_cursor};  # also available in __reset_mode
     if ( $self->{hide_cursor} ) {
-        print HIDE_CURSOR;
+        print hide_cursor();
     }
     my $mode_stty;
     if ( ! $config->{mode} ) {
@@ -192,46 +169,23 @@ sub __set_mode {
     else {
         die "Invalid mode!";
     }
-    if ( $Term_ReadKey ) {
+    if ( TERM_READKEY ) {
         Term::ReadKey::ReadMode( $config->{mode} );
     }
     else {
-        $Stty = `stty --save`;
+        $Stty = qx(stty --save);
         chomp $Stty;
         system( "stty -echo $mode_stty" ) == 0 or die $?;
     }
     if ( $self->{mouse} ) {
-        if ( $self->{mouse} == 3 ) {
-            my $return = binmode STDIN, ':utf8';
-            if ( $return ) {
-                print SET_ANY_EVENT_MOUSE_1003;
-                print SET_EXT_MODE_MOUSE_1005;
-            }
-            else {
-                $self->{mouse} = 0;
-                warn "binmode STDIN, :utf8: $!\nmouse-mode disabled\n";
-            }
-        }
-        elsif ( $self->{mouse} == 4 ) {
-            my $return = binmode STDIN, ':raw';
-            if ( $return ) {
-                print SET_ANY_EVENT_MOUSE_1003;
-                print SET_SGR_EXT_MODE_MOUSE_1006;
-            }
-            else {
-                $self->{mouse} = 0;
-                warn "binmode STDIN, :raw: $!\nmouse-mode disabled\n";
-            }
+        my $return = binmode STDIN, ':raw';
+        if ( $return ) {
+            print SET_ANY_EVENT_MOUSE_1003;
+            print SET_SGR_EXT_MODE_MOUSE_1006;
         }
         else {
-            my $return = binmode STDIN, ':raw';
-            if ( $return ) {
-                print SET_ANY_EVENT_MOUSE_1003;
-            }
-            else {
-                $self->{mouse} = 0;
-                warn "binmode STDIN, :raw: $!\nmouse-mode disabled\n";
-            }
+            $self->{mouse} = 0;
+            warn "binmode STDIN, :raw: $!\nmouse-mode disabled\n";
         }
     }
     return $self->{mouse};
@@ -242,12 +196,11 @@ sub __reset_mode {
     my ( $self ) = @_;
     if ( $self->{mouse} ) {
         binmode STDIN, ':encoding(UTF-8)' or warn "binmode STDIN, :encoding(UTF-8): $!\n";
-        print UNSET_EXT_MODE_MOUSE_1005     if $self->{mouse} == 3;
-        print UNSET_SGR_EXT_MODE_MOUSE_1006 if $self->{mouse} == 4;
+        print UNSET_SGR_EXT_MODE_MOUSE_1006;
         print UNSET_ANY_EVENT_MOUSE_1003;
     }
-    print RESET;
-    if ( $Term_ReadKey ) {
+    print normal();
+    if ( TERM_READKEY ) {
         Term::ReadKey::ReadMode( 'restore' );
     }
     else {
@@ -259,40 +212,51 @@ sub __reset_mode {
         }
     }
     if ( $self->{hide_cursor} ) {
-        print SHOW_CURSOR;
+        print show_cursor();
     }
 }
 
 
-sub __get_term_size {
+sub __get_cursor_row {
     #my ( $self ) = @_;
-    my ( $width, $height ) = ( 0, 0 );
-    if ( $Term_ReadKey ) {
-        ( $width, $height ) = ( Term::ReadKey::GetTerminalSize() )[ 0, 1 ];
-    }
-    else {
-        my $size = `stty size`;
-        if ( defined $size && $size =~ /(\d+)\s(\d+)/ ) {
-            $width  = $2;
-            $height = $1;
+    my $abs_curs_y;
+    print "\e[6n";
+    my $c1 = _getc_wrapper( 0 );
+    if ( defined $c1 && $c1 eq "\e" ) {
+        my $c2 = _getc_wrapper( 0.10 );
+        if ( defined $c2 && $c2 eq '[' ) {
+            my $c3 = _getc_wrapper( 0 );
+            if ( $c3 =~ m/^[0-9]$/ ) {
+                my $c4 = _getc_wrapper( 0 );
+                if ( $c4 =~ m/^[;0-9]$/ ) {
+                    my $curs_y = $c3;
+                    my $ry = $c4;
+                    while ( $ry =~ m/^[0-9]$/ ) {
+                        $curs_y .= $ry;
+                        $ry = _getc_wrapper( 0 );
+                    }
+                    if ( $ry eq ';' ) {
+                        my $curs_x = ''; # unused
+                        my $rx = _getc_wrapper( 0 );
+                        while ( $rx =~ m/^[0-9]$/ ) {
+                            $curs_x .= $rx;
+                            $rx = _getc_wrapper( 0 );
+                        }
+                        if ( $rx eq 'R' ) {
+                            $abs_curs_y = $curs_y;
+                        }
+                    }
+                }
+            }
         }
     }
-    return $width - WIDTH_CURSOR, $height;
+    return $abs_curs_y || 1;
 }
 
 
-sub __get_cursor_position {
-    my ( $self ) = @_;
-    #$self->{abs_cursor_x} = 0; # unused
-    $self->{abs_cursor_y} = 0;
-    print GET_CURSOR_POSITION;
-}
 
 
-sub __beep {
-    my ( $self, $beep ) = @_;
-    print BEEP if $beep;
-}
+
 
 
 
